from chaiverse.http_client import PrometheusClient
from chaiverse import config

from datetime import datetime

DEFAULT_QUERY_RANGE = 36000

class PrometheusService():
    def __init__(self,
            submission_id: str,
            start_epoch_time: int = None,
            end_epoch_time: int = None,
            step: str = "10m",
            developer_key: str = None):
        self.submission_id = submission_id
        self.start_epoch_time = start_epoch_time
        self.end_epoch_time = end_epoch_time
        self.step = step
        self.developer_key = developer_key

    @property
    def query_params(self):
        params = {
                "start": self.start_epoch_time,
                "end": self.end_epoch_time,
                "step": self.step
                }
        return params

    def get_metrics(self, endpoint):
        return self._make_prometheus_client_requests(endpoint)

    def get_usage_metrics(self):
        return self.get_metrics(config.USAGE_METRICS_ENDPOINT)

    def get_latency_metrics(self):
        return self.get_metrics(config.LATENCY_METRICS_ENDPOINT)

    def _make_prometheus_client_requests(self, endpoint):
        http_client = PrometheusClient(developer_key=self.developer_key)
        response = http_client.get(endpoint=endpoint, params=self.query_params, submission_id=self.submission_id)
        return response




import sys

# check if the symbol is already imported to avoid recursive import error
if 'developer_login' not in sys.modules:
    from chaiverse.cli_utils.login_cli import developer_login

import chaiverse.formatters as formatters

from chaiverse.chat import SubmissionChatbot

from chaiverse.submit import (
    ModelSubmitter,
    deactivate_model,
    redeploy_model,
    evaluate_model,
    get_model_info,
    get_my_submissions,
)

from chaiverse.feedback import get_feedback

from chaiverse.leaderboard_cli import (
    display_leaderboard,
    get_leaderboard,
    get_leaderboards,
    auto_deactivate,
)

from chaiverse.feed import get_feed

from chaiverse.competition_cli import (
    get_competitions,
    create_competition,
    update_competition,
    enroll_submission,
    withdraw_submission,
)

from chaiverse.prometheus_metrics import PrometheusService



from datetime import datetime, timezone
import dateparser
import pytz
from typing import Optional, Literal

from chaiverse.schemas.date_range_schema import DateRange

US_PACIFIC = pytz.timezone('US/Pacific')
UTC = pytz.timezone('UTC')


def is_epoch_time_before_date_range(epoch_time, date_range: Optional[DateRange]):
    is_before = date_range and epoch_time < date_range.start_epoch_time
    return is_before


def is_epoch_time_in_date_range(epoch_time, date_range: Optional[DateRange]):
    is_in = True
    if date_range:
        is_in = date_range.start_epoch_time <= epoch_time < date_range.end_epoch_time
    return is_in


def us_pacific_date_string() -> str:
    date = datetime.now(US_PACIFIC)
    date_string = date.strftime('%Y-%m-%d')
    return date_string


def us_pacific_string(date_string: Optional[str]=None):
    return _create_date_string_in_timezone(US_PACIFIC, date_string)


def utc_string(date_string: Optional[str]=None):
    return _create_date_string_in_timezone(UTC, date_string)


def convert_to_utc_iso_format(date_string: Optional[str], default_timezone=US_PACIFIC):
    date_string_in_utc = None
    if date_string is not None:
        date = datetime.fromisoformat(date_string)
        date = default_timezone.localize(date) if not date.tzinfo else date
        date_in_utc = date.astimezone(timezone.utc)
        date_string_in_utc = date_in_utc.isoformat()
    return date_string_in_utc


def convert_to_us_pacific_date(date: Optional[datetime], default_timezone=UTC):
    date_in_us_pacific = None
    if date is not None:
        date = default_timezone.localize(date) if not date.tzinfo else date
        date_in_us_pacific = date.astimezone(US_PACIFIC)
    return date_in_us_pacific


def _create_date_string_in_timezone(pytz_timezone, date_string: Optional[str]=None):
    if date_string:
        date = dateparser.parse(date_string)
        date = pytz_timezone.localize(date)
    else:
        date = datetime.now(pytz_timezone)
    return date.isoformat()


def _get_date_range_field(date_range=None, field=Literal['start_date', 'end_date']):
    epoch_time = None
    if date_range and date_range.get(field):
        isoformat_date = date_range.get(field)
        date = datetime.fromisoformat(isoformat_date) 
        assert date.tzinfo
        epoch_time = date.timestamp()
    return epoch_time



BASE_SUBMITTER_URL = "https://guanaco-submitter.chai-research.com"
BASE_FEEDBACK_URL = "https://guanaco-feedback.chai-research.com"
BASE_PROMETHEUS_URL = "https://guanaco-prometheus.chai-research.com"
BASE_AUTH_URL = "https://auth.chaiverse.com"

LATEST_LEADERBOARD_ENDPOINT = "/latest_leaderboard"
LEADERBOARDS_ENDPOINT = "/leaderboards"
LEADERBOARD_AUTO_DEACTIVATE = '/auto_deactivate'
LEADERBOARD_ENDPOINT = "/leaderboard"
CHAT_ENDPOINT = "/models/{submission_id}/chat"
FEEDBACK_SUMMARY_ENDPOINT = "/feedback"
FEEDBACK_ENDPOINT = "/feedback/{submission_id}"

SUBMISSION_ENDPOINT = "/models/submit"
BLEND_SUBMISSION_ENDPOINT = "/models/submit_blend"
REWARD_BLEND_SUBMISSION_ENDPOINT = "/models/submit_reward_blend"
ROUTED_BLEND_SUBMISSION_ENDPOINT = "/models/submit_routed_blend"
ALL_SUBMISSION_STATUS_ENDPOINT = "/models/"
SEARCH_SUBMISSIONS_ENDPOINT = "/models/search"
INFO_ENDPOINT = "/models/{submission_id}"
DEACTIVATE_ENDPOINT = "/models/{submission_id}/deactivate"
REDEPLOY_ENDPOINT = "/models/{submission_id}/redeploy"
EVALUATE_ENDPOINT = "/models/{submission_id}/evaluate"
TEARDOWN_ENDPOINT = "/models/{submission_id}/teardown"
FEED_ENDPOINT = "/feed"
USER_FEED_ENDPOINT = "/users/{username}/feed"

COMPETITIONS_ENDPOINT = '/competitions'
COMPETITION_ENDPOINT = '/competitions/{competition_id}'
COMPETITION_ENROLLED_SUBMISSION_IDS_ENDPOINT = '/competitions/{competition_id}/enrolled_submission_ids/{submission_id}'

USAGE_METRICS_ENDPOINT = '/{submission_id}/usage-metrics'
LATENCY_METRICS_ENDPOINT = '/{submission_id}/latency-metrics'

DEFAULT_BEST_OF = 4
DEFAULT_REWARD_REPO = "ChaiML/reward_gpt2_medium_preference_24m_e2"
DEFAULT_MAX_INPUT_TOKENS = 512
DEFAULT_MAX_OUTPUT_TOKENS = 64

AUTO_DEACTIVATION_MIN_NUM_BATTLES = 5000
AUTO_DEACTIVATION_MIN_NUM_PROPRIETY = 5000
AUTO_DEACTIVATION_MAX_ELO_RATING = 1000
AUTO_DEACTIVATION_MIN_RANK = 1

LEADERBOARD_STABLE_ELO_REQUIRED_BATTLES = 5000
LEADERBOARD_REQUIRED_PROPRIETY = 5000

ELO_REQUIRED_BATTLES = 1000
ELO_BASE_SUBMISSION_ID = 'mistralai-mixtral-8x7b-_3473_v11'
ELO_BASE_RATING = 1114


def get_elo_base_submission_id():
    return ELO_BASE_SUBMISSION_ID


def get_elo_base_rating():
    return ELO_BASE_RATING
